import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:sdk_example/repositories/auth_repository.dart';
import 'package:sdk_example/repositories/transaction_repository.dart';
import 'package:sdk_example/repositories/wallet_repository.dart';
import 'package:sdk_example/screens/auth_screens/phone_number_auth_screen.dart';
import 'package:m4e_sdk/m4e_sdk.dart';
import 'package:sdk_example/screens/splash_screen.dart';

void main() {
  WidgetsFlutterBinding.ensureInitialized();

  // initialize m4e-sdk
  initializeM4eSDK(
      config: M4eSDKConfig.staging(appId: M4eUniqueId().toString()));
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: [
        ChangeNotifierProvider<AuthRepository>(create: (_) => AuthRepository()),
        ChangeNotifierProvider<WalletRepository>(
          create: (_) => WalletRepository(),
        ),
        ChangeNotifierProvider<TransactionRepository>(
          create: (_) => TransactionRepository(),
        ),
      ],
      builder: (context, _) {
        return MaterialApp(
          title: 'M4e SDK',
          theme: ThemeData(
            primarySwatch: Colors.indigo,
            accentColor: Colors.amber,
            visualDensity: VisualDensity.adaptivePlatformDensity,
          ),
          // home: PhoneNumberAuthScreen(),
          home: SplashScreen(),
        );
      },
    );
  }
}
