import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter_secured_storage/flutter_secured_storage.dart';
import 'package:sdk_example/services/transaction_service.dart';
import 'package:m4e_sdk/m4e_sdk.dart';

enum TransactionRepositoryState { initial, loading, success, error }

class TransactionRepository with ChangeNotifier {
  final _transactionService = TransactionService();
  final _storage = FlutterSecuredStorage();

  static M4eTransactionList _transactionList;
  M4eTransactionList get transactionList => _transactionList;

  static M4eException _error;
  M4eException get error => _error;

  static TransactionRepositoryState _state;
  TransactionRepositoryState get state =>
      _state ?? TransactionRepositoryState.initial;

  Future<void> getUserTransactions() async {
    try {
      _state = TransactionRepositoryState.loading;
      notifyListeners();

      // get user from cache
      final _userString = await _storage.read(key: 'M4E_USER');

      if (_userString != null) {
        final _m4eUser = M4eUser.fromJson(json.decode(_userString));

        _transactionList =
            await _transactionService.getUserTransactions(_m4eUser.id);

        _state = TransactionRepositoryState.success;
        notifyListeners();
      } else {
        _error = M4eExceptionMessages.kUnauthorized;

        _state = TransactionRepositoryState.error;
        notifyListeners();
      }
    } on M4eException catch (e) {
      _error = e;

      _state = TransactionRepositoryState.error;
      notifyListeners();
    }
  }

  Future<void> createTransaction(
      {@required M4eTransactionForm transactionForm}) async {
    try {
      _state = TransactionRepositoryState.loading;
      notifyListeners();

      // get user from cache
      final _userString = await _storage.read(key: 'M4E_USER');

      if (_userString != null) {
        final _m4eUser = M4eUser.fromJson(json.decode(_userString));

        await _transactionService.createTransactions(
            _m4eUser.id, transactionForm);

        _state = TransactionRepositoryState.success;
        notifyListeners();
      } else {
        _error = M4eExceptionMessages.kUnauthorized;

        _state = TransactionRepositoryState.error;
        notifyListeners();
      }
    } on M4eException catch (e) {
      _error = e;

      _state = TransactionRepositoryState.error;
      notifyListeners();
    }
  }
}
