import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:provider/provider.dart';
import 'package:sdk_example/repositories/auth_repository.dart';
import 'package:m4e_sdk/m4e_sdk.dart';
import 'package:sdk_example/screens/home_screen.dart';

class UserAccountScreen extends StatefulWidget {
  @override
  _UserAccountScreenState createState() => _UserAccountScreenState();
}

class _UserAccountScreenState extends State<UserAccountScreen> {
  @override
  void initState() {
    super.initState();

    Future.microtask(() => _getUserAccount());
  }

  void _getUserAccount() {
    final _repository = Provider.of<AuthRepository>(context, listen: false);

    _repository.getAuthentiicatedUserAccount().then((_) {
      if (_repository.state == AuthRepositoryState.error) {
        Fluttertoast.showToast(
            msg: '${_repository.error.code}\n${_repository.error.message}');

        if (_repository.error.code == M4eExceptionCode.kNotFoundExceptionCode) {
          // navigate to home screen
          Navigator.of(context).pushReplacement(
              MaterialPageRoute(builder: (context) => UserAccountFormScreen()));
        }
      } else {
        // navigate to home screen
        Navigator.of(context).pushReplacement(
            MaterialPageRoute(builder: (context) => HomeScreen()));
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SafeArea(
        child: Consumer<AuthRepository>(
          builder: (_, repository, __) {
            if (repository.state == AuthRepositoryState.error) {
              return Center(
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Text(
                        '${repository.error.code}\n${repository.error.message}'),
                    RaisedButton(
                      onPressed: () => _getUserAccount(),
                      child: Text('Retry'),
                    )
                  ],
                ),
              );
            }

            return Center(child: CircularProgressIndicator());
          },
        ),
      ),
    );
  }
}

class UserAccountFormScreen extends StatelessWidget {
  final _formKey = GlobalKey<FormState>();

  final _firstNameController = TextEditingController();
  final _lastNameController = TextEditingController();
  final _phoneNumberController = TextEditingController();

  // TODO: properlly implement and test
  void _createUserAccount(BuildContext context) {
    final _repository = Provider.of<AuthRepository>(context, listen: false);

    if (!_formKey.currentState.validate()) return;

    final _m4eUserForm = M4eUserForm(
      firstName: _firstNameController.text,
      lastName: _lastNameController.text,
      phoneNumber: M4ePhoneNumber(
        country: 'GHANA', // hard-coded value
        number: _phoneNumberController.text,
      ),
    );

    _repository.createUserAccount(m4eUserForm: _m4eUserForm).then((_) {
      if (_repository.state == AuthRepositoryState.error) {
        Fluttertoast.showToast(
            msg: '${_repository.error.code}\n${_repository.error.message}');
      } else {
        // navigate to sign-up screen
        Navigator.of(context).pushReplacement(
            MaterialPageRoute(builder: (context) => UserAccountScreen()));
      }
    });
  }

  Widget _buildFirstnameTextField() {
    return TextFormField(
      controller: _firstNameController,
      textAlignVertical: TextAlignVertical.center,
      decoration: InputDecoration(
        hintText: 'Enter firstname',
      ),
      validator: (_) {
        return _firstNameController.text.isEmpty ? 'Enter firstname' : null;
      },
    );
  }

  Widget _buildLastnameTextField() {
    return TextFormField(
      controller: _lastNameController,
      textAlignVertical: TextAlignVertical.center,
      decoration: InputDecoration(
        hintText: 'Enter lastname',
      ),
      validator: (_) {
        return _lastNameController.text.isEmpty ? 'Enter lastname' : null;
      },
    );
  }

  Widget _buildPhoneNumberTextField() {
    return TextFormField(
      controller: _phoneNumberController,
      textAlignVertical: TextAlignVertical.center,
      decoration: InputDecoration(
        hintText: 'Enter phone number',
      ),
      validator: (_) {
        return _phoneNumberController.text.isEmpty
            ? 'Enter phone number'
            : null;
      },
    );
  }

  Widget _buildSubmitButton() {
    return Consumer<AuthRepository>(
      builder: (context, repository, __) {
        return FloatingActionButton(
          onPressed: () => _createUserAccount(context),
          child: repository.state == AuthRepositoryState.loading
              ? CircularProgressIndicator(
                  backgroundColor: Theme.of(context).primaryColor)
              : Icon(Icons.arrow_forward_ios),
        );
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SafeArea(
        child: Form(
          key: _formKey,
          child: ListView(
            padding: EdgeInsets.symmetric(horizontal: 30.0),
            children: [
              SizedBox(height: 50.0),
              Text(
                'Create Account',
                style: Theme.of(context)
                    .primaryTextTheme
                    .headline4
                    .copyWith(color: Colors.black),
              ),
              _buildFirstnameTextField(),
              SizedBox(height: 20.0),
              _buildLastnameTextField(),
              SizedBox(height: 20.0),
              _buildPhoneNumberTextField(),
              SizedBox(height: 30.0),
              _buildSubmitButton(),
              SizedBox(height: 30.0),
            ],
          ),
        ),
      ),
    );
  }
}
