import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:provider/provider.dart';
import 'package:sdk_example/repositories/auth_repository.dart';
import 'package:sdk_example/screens/auth_screens/phone_number_auth_screen.dart';
import 'package:sdk_example/screens/auth_screens/user_account_screen.dart';

class SplashScreen extends StatefulWidget {
  @override
  _SplashScreenState createState() => _SplashScreenState();
}

class _SplashScreenState extends State<SplashScreen> {
  @override
  void initState() {
    super.initState();

    Future.microtask(() => _verifyAuthenticationStatus());
  }

  void _verifyAuthenticationStatus() {
    final _repository = Provider.of<AuthRepository>(context, listen: false);

    _repository.isAuthenticated().then((_) {
      if (_repository.state == AuthRepositoryState.error) {
        Fluttertoast.showToast(
            msg: '${_repository.error.code}\n${_repository.error.message}');
      } else {
        if (_repository.m4eAuthToken == null) {
          // navigate to phone authentication screen
          Navigator.of(context).pushReplacement(
              MaterialPageRoute(builder: (context) => PhoneNumberAuthScreen()));
        } else {
          // navigate to user account screen to fetch profile
          Navigator.of(context).pushReplacement(
              MaterialPageRoute(builder: (context) => UserAccountScreen()));
        }
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Center(
        child: CircularProgressIndicator(),
      ),
    );
  }
}
