import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:provider/provider.dart';
import 'package:sdk_example/repositories/transaction_repository.dart';
import 'package:m4e_sdk/m4e_sdk.dart';
import 'package:sdk_example/screens/transaction_screens/transaction_form_screen.dart';

class TransactionsScreen extends StatefulWidget {
  @override
  _TransactionsScreenState createState() => _TransactionsScreenState();
}

class _TransactionsScreenState extends State<TransactionsScreen> {
  @override
  void initState() {
    super.initState();

    Future.microtask(() => _getUserTransactions());
  }

  void _navigateToCreateTransactionScreen() {
    Navigator.of(context)
        .push(MaterialPageRoute(builder: (context) => TransactionFormScreen()))
        .then((value) => _getUserTransactions());
  }

  void _getUserTransactions() {
    final _repository =
        Provider.of<TransactionRepository>(context, listen: false);

    _repository.getUserTransactions().then((_) {
      if (_repository.state == TransactionRepositoryState.error) {
        Fluttertoast.showToast(
            msg: '${_repository.error.code}\n${_repository.error.message}');
      }
    });
  }

  Widget _buildMakeTransactionFloatingActionButton() {
    return FloatingActionButton(
      onPressed: () => _navigateToCreateTransactionScreen(),
      child: Icon(Icons.add),
    );
  }

  Widget _buildTransactionList(M4eTransactionList transactionList) {
    return ListView.builder(
      itemCount: transactionList.transactions.length,
      itemBuilder: (context, index) {
        final _transaction = transactionList.transactions[index];

        return ListTile(
          onTap: () {},
          contentPadding: EdgeInsets.symmetric(horizontal: 30.0),
          title: Text(
              '${_transaction.from.walletInfo.label} - ${_transaction.to.walletInfo.label}'),
          trailing: Text(
              '${_transaction.amount.currency} ${_transaction.amount.value}'),
        );
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      floatingActionButton: _buildMakeTransactionFloatingActionButton(),
      appBar: AppBar(
        title: Text('Transactions'),
      ),
      body: Consumer<TransactionRepository>(
        builder: (_, repository, __) {
          if (repository.state == TransactionRepositoryState.error) {
            return Center(
              child: Padding(
                padding: const EdgeInsets.symmetric(horizontal: 30.0),
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Text(
                        '${repository.error.code}\n${repository.error.message}'),
                    RaisedButton(
                      onPressed: () => _getUserTransactions(),
                      child: Text('Retry'),
                    ),
                  ],
                ),
              ),
            );
          }

          if (repository.state == TransactionRepositoryState.success &&
              repository.transactionList != null) {
            return _buildTransactionList(repository.transactionList);
          }

          return Center(
            child: CircularProgressIndicator(),
          );
        },
      ),
    );
  }
}
