import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../core.dart';

abstract class M4eException extends Equatable implements Exception {
  final M4eExceptionCode code;
  final String message;

  const M4eException({@required this.code, @required this.message});

  @override
  List<Object> get props => [code, message];
}

/// This is thrown when a an `M4eServerException` occures
class M4eServerException extends M4eException {
  const M4eServerException(
      {@required M4eExceptionCode code, @required String message})
      : super(code: code, message: message);

  factory M4eServerException.fromJson(Map<String, dynamic> json) {
    return M4eServerException(
      code: M4eExceptionCode.kUndefinedExceptionCode,
      message: json['error'],
    );
  }

  factory M4eServerException.fromString(String exception) {
    return M4eServerException(
      code: M4eExceptionCode.kUndefinedExceptionCode,
      message: exception,
    );
  }

  Map<String, dynamic> toJson() {
    return {
      'error': this.message,
    };
  }

  @override
  String toString() {
    return '[M4eServerException] code: ${code.toString()}, message:$message';
  }
}

/// This is thrown when a an `M4eCacheException` occures
class M4eCacheException extends M4eException {
  const M4eCacheException(
      {@required M4eExceptionCode code, @required String message})
      : super(code: code, message: message);

  @override
  String toString() {
    return '[M4eCacheException] code: ${code.toString()}, message:$message';
  }
}

/// This is thrown when `M4eAuthException` occures
class M4eAuthException extends M4eException {
  const M4eAuthException(
      {@required M4eExceptionCode code, @required String message})
      : super(code: code, message: message);

  @override
  String toString() {
    return '[M4eAuthException] code: ${code.toString()}, message:$message';
  }
}
