import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../../core/core.dart';
import '../../models/models.dart';
import '../../m4e_forms/m4e_forms.dart';

/// Form parameters required for m4e-sdk transactions
///
class M4eTransactionForm extends Equatable {
  final M4eWallet from;
  final M4eReciepientWallet to;
  final M4eAmount amount;
  final DateTime at;
  final Map<String, dynamic> meta;

  const M4eTransactionForm._({
    @required this.from,
    @required this.to,
    @required this.amount,
    @required this.at,
    @required this.meta,
  });

  factory M4eTransactionForm({
    @required M4eWallet from,
    @required M4eReciepientWallet to,
    @required M4eAmount amount,
    @required DateTime at,
    @required Map<String, dynamic> meta,
  }) {
    assert(from != null, 'M4eTransactionForm [from] wallet should not be null');
    assert(to != null, 'M4eTransactionForm [to] wallet should not be null');
    assert(
        from.id !=
            to.maybeWhen(
              m4eWalletProvider: (wallet) => wallet.id,
              orElse: () => null,
            ),
        'M4eTransactionForm [from] and [to] wallets should not be the same');
    assert(amount != null, 'M4eTransactionForm [amount] should not be null');
    assert(at != null, 'M4eTransactionForm [at] should not be null');
    assert(meta != null, 'M4eTransactionForm [meta] should not be null');

    return M4eTransactionForm._(
        from: from, to: to, amount: amount, at: at, meta: meta);
  }

  @override
  List<Object> get props => [
        from,
        to,
        amount,
        at,
        meta,
      ];

  @override
  String toString() {
    return 'M4eTransactionForm(from: $from, to: ${to.toString()}, amount: $amount, at: $at, meta: $meta)';
  }
}
